@@ -122,7 +122,7 @@ class WheelArchive:
 
 class WheelBuilderConfig(BuilderConfig):
     def __init__(self, *args, **kwargs):
-        super(WheelBuilderConfig, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         self.__include_defined = bool(
             self.target_config.get('include', self.build_config.get('include'))
@@ -136,6 +136,7 @@ class WheelBuilderConfig(BuilderConfig):
         self.__core_metadata_constructor = None
         self.__shared_data = None
         self.__extra_metadata = None
+        self.__strict_naming = None
 
     def set_default_file_selection(self):
         if self.__include or self.__exclude or self.__packages:
@@ -255,6 +256,24 @@ class WheelBuilderConfig(BuilderConfig):
 
         return self.__extra_metadata
 
+    @property
+    def strict_naming(self):
+        if self.__strict_naming is None:
+            if 'strict-naming' in self.target_config:
+                strict_naming = self.target_config['strict-naming']
+                if not isinstance(strict_naming, bool):
+                    raise TypeError(
+                        f'Field `tool.hatch.build.targets.{self.plugin_name}.strict-naming` must be a boolean'
+                    )
+            else:
+                strict_naming = self.build_config.get('strict-naming', True)
+                if not isinstance(strict_naming, bool):
+                    raise TypeError('Field `tool.hatch.build.strict-naming` must be a boolean')
+
+            self.__strict_naming = strict_naming
+
+        return self.__strict_naming
+
 
 class WheelBuilder(BuilderInterface):
     """
@@ -285,7 +304,9 @@ class WheelBuilder(BuilderInterface):
             else:
                 build_data['tag'] = self.get_default_tag()
 
-        with WheelArchive(self.project_id, self.config.reproducible) as archive, closing(StringIO()) as records:
+        with WheelArchive(self.artifact_project_id, self.config.reproducible) as archive, closing(
+            StringIO()
+        ) as records:
             for included_file in self.recurse_included_files():
                 record = archive.add_file(included_file)
                 records.write(self.format_record(record))
@@ -295,7 +316,7 @@ class WheelBuilder(BuilderInterface):
             records.write(f'{archive.metadata_directory}/RECORD,,\n')
             archive.write_metadata('RECORD', records.getvalue())
 
-        target = os.path.join(directory, f"{self.project_id}-{build_data['tag']}.whl")
+        target = os.path.join(directory, f"{self.artifact_project_id}-{build_data['tag']}.whl")
 
         replace_file(archive.path, target)
         return target
@@ -309,7 +330,9 @@ class WheelBuilder(BuilderInterface):
     def build_editable_detection(self, directory, **build_data):
         build_data['tag'] = self.get_default_tag()
 
-        with WheelArchive(self.project_id, self.config.reproducible) as archive, closing(StringIO()) as records:
+        with WheelArchive(self.artifact_project_id, self.config.reproducible) as archive, closing(
+            StringIO()
+        ) as records:
             exposed_packages = {}
             for included_file in self.recurse_project_files():
                 if not included_file.path.endswith('.py'):
@@ -330,10 +353,17 @@ class WheelBuilder(BuilderInterface):
                     exposed_packages[root_module] = os.path.join(self.root, root_module)
                 else:
                     distribution_module = distribution_path.split(os.sep)[0]
-                    exposed_packages[distribution_module] = os.path.join(
-                        self.root,
-                        f'{relative_path[:relative_path.index(distribution_path)]}{distribution_module}',
-                    )
+                    try:
+                        exposed_packages[distribution_module] = os.path.join(
+                            self.root,
+                            f'{relative_path[:relative_path.index(distribution_path)]}{distribution_module}',
+                        )
+                    except ValueError:
+                        raise ValueError(
+                            'Dev mode installations are unsupported when any path rewrite in the `sources` option '
+                            'changes a prefix rather than removes it, see: '
+                            'https://github.com/pfmoore/editables/issues/20'
+                        ) from None
 
             editable_project = EditableProject(self.metadata.core.name, self.root)
 
@@ -366,7 +396,7 @@ class WheelBuilder(BuilderInterface):
             records.write(f'{archive.metadata_directory}/RECORD,,\n')
             archive.write_metadata('RECORD', records.getvalue())
 
-        target = os.path.join(directory, f"{self.project_id}-{build_data['tag']}.whl")
+        target = os.path.join(directory, f"{self.artifact_project_id}-{build_data['tag']}.whl")
 
         replace_file(archive.path, target)
         return target
@@ -374,7 +404,9 @@ class WheelBuilder(BuilderInterface):
     def build_editable_explicit(self, directory, **build_data):
         build_data['tag'] = self.get_default_tag()
 
-        with WheelArchive(self.project_id, self.config.reproducible) as archive, closing(StringIO()) as records:
+        with WheelArchive(self.artifact_project_id, self.config.reproducible) as archive, closing(
+            StringIO()
+        ) as records:
             directories = sorted(
                 os.path.normpath(os.path.join(self.root, relative_directory))
                 for relative_directory in self.config.dev_mode_dirs
@@ -392,7 +424,7 @@ class WheelBuilder(BuilderInterface):
             records.write(f'{archive.metadata_directory}/RECORD,,\n')
             archive.write_metadata('RECORD', records.getvalue())
 
-        target = os.path.join(directory, f"{self.project_id}-{build_data['tag']}.whl")
+        target = os.path.join(directory, f"{self.artifact_project_id}-{build_data['tag']}.whl")
 
         replace_file(archive.path, target)
         return target
@@ -421,7 +453,7 @@ class WheelBuilder(BuilderInterface):
         # entry_points.txt
         self.write_entry_points_file(archive, records)
 
-        # license_files/
+        # licenses/
         self.add_licenses(archive, records)
 
         # extra_metadata/ - write last
@@ -458,7 +490,7 @@ Root-Is-Purelib: {'true' if build_data['pure_python'] else 'false'}
         for relative_path in self.metadata.core.license_files:
             license_file = os.path.normpath(os.path.join(self.root, relative_path))
             with open(license_file, 'rb') as f:
-                record = archive.write_metadata(f'license_files/{relative_path}', f.read())
+                record = archive.write_metadata(f'licenses/{relative_path}', f.read())
                 records.write(self.format_record(record))
 
     def add_extra_metadata(self, archive, records):
@@ -507,6 +539,14 @@ Root-Is-Purelib: {'true' if build_data['pure_python'] else 'false'}
 
         return normalize_inclusion_map(build_data['force_include_editable'], self.root)
 
+    @property
+    def artifact_project_id(self):
+        return (
+            self.project_id
+            if self.config.strict_naming
+            else f'{self.normalize_file_name_component(self.metadata.core.raw_name)}-{self.metadata.version}'
+        )
+
     @classmethod
     def get_config_class(cls):
         return WheelBuilderConfig
